#!/bin/bash

# read user customization if any, otherwise use default values
BURNSETUPCONF=/etc/vdr/plugins/burn/vdrburnscript.conf
if [ "!" -f "$BURNSETUPCONF" ]; then
    echo ""
    echo "You could customize '$0'"
    echo "in '$BURNSETUPCONF'"
    echo "and that way keep your settings between releases."
    echo "A sample file is provided in the source package"
    echo "in the 'PLUGINS/src/burn/burn' subdirectory."
    echo ""
else
    # we found a file, we trust the user :-)
    source $BURNSETUPCONF
fi

# Length in minutes of each chapter when DVD without menues is created
CHAPTER_LEN="${CHAPTER_LEN:-5}"

# Start Command with nice ? If not wished then set NICE=""
NICE="${NICE:-nice -n 19}"

# Use cut function of vdrsync (yes|no)
USE_CUT="${USE_CUT:-yes}"

# Use "tcrequant" or plain "requant" command (yes|no)
USE_TCREQUANT="${USE_TCREQUANT:-yes}"

# Use "tcmplex" or "mplex" command (yes|no)
USE_TCMPLEX="${USE_TCMPLEX:-no}"

# Script version
PLUGIN_VERSION="0.0.009"

# when script is called out of $PATH then add directory of script to $PATH
# there may be other needed programs in this directory
export PATH=$PATH:$(dirname $0)

logger -s "<$*>"

IsExecutable() {
    if type "$1" >/dev/null 2>&1; then
        return 0
    fi
    return 1
}

DetectTcrequant() {
    if IsExecutable tcrequant; then
        USE_TCREQUANT=yes
    else
        if [ "$USE_TCREQUANT" != "yes" ]; then
            logger "Unable to find valid requant program."
            exit 1
        fi
        DetectRequant
    fi
}

DetectRequant() {
    if IsExecutable requant; then
        USE_TCREQUANT=no
    else
        if [ "$USE_TCREQUANT" == "yes" ]; then
            logger "Unable to find valid requant program."
            exit 1
        fi
        DetectTcrequant
    fi
}

DetectRequanter() {
    if [ "$USE_TCREQUANT" == "yes" ]; then
        DetectTcrequant
    else
        DetectRequant
    fi
}

DetectTcmplex() {
    if IsExecutable tcmplex-panteltje; then
        TCMPLEX=tcmplex-panteltje
        USE_TCMPLEX=yes
    elif IsExecutable tcmplex; then
        TCMPLEX=tcmplex
        USE_TCMPLEX=yes
    else
        # check if we already are the fallback
        if [ "$USE_TCMPLEX" != "yes" ]; then
            logger "Unable to find valid mplex program."
            exit 1
        fi
        DetectMplex
    fi
}

DetectMplex() {
    if IsExecutable mplex; then
        TCMPLEX=mplex
        USE_TCMPLEX=no
    else
        # check if we already are the fallback
        if [ "$USE_TCMPLEX" == "yes" ]; then
            logger "Unable to find valid mplex program."
            exit 1
        fi
        DetectTcmplex
    fi
}

DetectMplexer() {
    if [ "$USE_TCMPLEX" == "yes" ]; then
        DetectTcmplex
    else
        DetectMplex
    fi
}

Png2Yuv() {
    if IsExecutable y4mscaler; then
        convert "$1" pnm:- |
            ppmtoy4m -n 1 -F25:1 -A 59:54 -I t -L -r -v 0 |
            y4mscaler -O chromass=420_MPEG2
    elif IsExecutable png2yuv; then
        png2yuv -n1 -f25 -It -L1 -v0 -j "$1"
    else
        convert "$1" pnm:- |
            ppmtoy4m -n 1 -F25:1 -A 59:54 -I t -L -r -v 0
        logger "Unable to find valid png to yuv converter program - trying to work anyway"
    fi
}

ExecCmd() {
    logger -s "Starting <$NICE $*>"
    $NICE "$@"
}

CheckCmd() {
    if IsExecutable "$1"; then
        echo "Unable to find '$1'."
        return 1
    fi
    return 0
}

# debug script execution
#set -x

DetectMplexer
DetectRequanter

# Sync Syntax: TempDir Number FileName
case "${1}" in
    SYNC)
        if [ "$USE_CUT" == "yes" ]; then
            CUTCMD="-cut"
        fi
        ExecCmd vdrsync.pl $CUTCMD -o "$2/VDRSYNC.$3" "$4"
        ;;

    # Requant Syntax: TempDir Number Filename Faktor
    REQUANT)
        if [ "$USE_TCREQUANT" == "yes" ]; then
            ExecCmd tcrequant -f "$5" -i "$4" -o "$2/VDRSYNC.$3/requant.mpv"
        else
            ExecCmd requant "$5" < "$4" > "$2/VDRSYNC.$3/requant.mpv"
        fi
        mv "$2/VDRSYNC.$3/requant.mpv" "$2/VDRSYNC.$3/vdrsync.mpv" 
        ;;

    # Mplex Syntax: TempDir FileName1 FileName2 FileName3
    MPLEX)
        if [ "$USE_TCMPLEX" == "yes" ]; then
            if [ "$5" != "" ]; then
                ExecCmd $TCMPLEX -i "$3" -p "$4" -s "$5" -m d -o "$2/movie.mpg"
            else
                ExecCmd $TCMPLEX -i "$3" -p "$4" -m d -o "$2/movie.mpg"
            fi
        else
            if [ "$5" != "" ]; then
                ExecCmd mplex -f 8 -vbr -v 1 -o "$2/movie.mpg" "$3" "$4" "$5"
            else
                ExecCmd mplex -f 8 -vbr -v 1 -o "$2/movie.mpg" "$3" "$4"
            fi
        fi
        ;;

    MKMENU)
        if [ "$USE_TCMPLEX" == "yes" ]; then
            Png2Yuv "$2/menu-bg-$3.png" |
                mpeg2enc -q 2 -a 2 -n p -f 8 -v 0 -o "$2/menu-bg-$3.m2v" &&
                $TCMPLEX -i "$2/menu-bg-$3.m2v" -p "$4/menu-silence.mp2" -m d -o /dev/stdout |
                spumux -v 2 "$2/menu-$3.xml" > "$2/menu-bg-$3.mpg"
        else
            Png2Yuv "$2/menu-bg-$3.png" |
                mpeg2enc -q 2 -a 2 -n p -f 8 -v 0 -o "$2/menu-bg-$3.m2v" &&
                mplex -f 8 -vbr -v 0 -o /dev/stdout "$2/menu-bg-$3.m2v" "$4/menu-silence.mp2" |
                spumux -v 2 "$2/menu-$3.xml" > "$2/menu-bg-$3.mpg"
        fi
        ;;

    # Author Syntax: TempDir or XML-File
    AUTHOR)
        if [ -d "$2" ]; then
            cd "$2"
            for i in $(seq 0 99) ; do
                if [ -f "VDRSYNC.$i/movie.mpg" ] ; then
                    echo "Found: VDRSYNC.$i"
                    CS="-c 0"
                    for j in $(seq $CHAPTER_LEN $CHAPTER_LEN 300) ; do
                        SH=$(($j / 60))
                        SM=$(($j % 60))
                        CS="$CS,$SH:$SM:0"
                    done
                    ExecCmd dvdauthor -o "$2/DVDAUTHOR" $CS -f "$2/VDRSYNC.$i/movie.mpg"
                else
                    break
                fi
            done
            ExecCmd dvdauthor -T -o "$2/DVDAUTHOR"
        else
            ExecCmd dvdauthor -x "$2"
        fi
        ;;

    # AUTHORONLY Syntax: TempDir IsoDir JobTitle
    AUTHORONLY)
	mv -gf "$2/DVDAUTHOR" "$3/$4"
	chmod -R 0777 "$3/$4"
        ;;

    # MkIso Syntax: TempDir IsoDir JobTitle
    MKISO)
        ExecCmd mkisofs -V "${4:0:32}" -dvd-video "$2/DVDAUTHOR" > "$3/$4.iso"
        ;;

    # Burn Syntax: TempDir Device JobTitle
    BURN)
        ExecCmd growisofs -use-the-force-luke=tty -dvd-compat -Z "$3" -V "${4:0:32}" -dvd-video "$2/DVDAUTHOR"
        ;;

    # MkIso-Burn Syntax: TempDir Device IsoDir JobTitle FifoPath
    MKISO_BURN)
        ExecCmd growisofs -use-the-force-luke=tty -dvd-compat -Z "$3=$6" | mkisofs -V "${5:0:32}" -dvd-video "$2/DVDAUTHOR" | tee "$4/$5.iso" > "$6"
        ;;

    CHECK)
#        if [ "$2" != "$PLUGIN_VERSION" ]; then
#            echo "Invalid plugin version $2 != $PLUGIN_VERSION."
#            exit
#        fi  
        CheckCmd growisofs &&
        CheckCmd mkisofs &&
        CheckCmd dvdauthor &&
        CheckCmd png2yuv &&
        CheckCmd ppmtoy4m &&
        CheckCmd y4mscaler &&
        CheckCmd mpeg2enc &&
        CheckCmd spumux &&
        CheckCmd vdrsync.pl
        if [ "$?" -eq 0 ]; then
            if [ "$USE_TCMPLEX" == "yes" ]; then
                CheckCmd $TCMPLEX
            else
                CheckCmd mplex
            fi
            if [ "$?" -eq 0 ]; then
                echo "Ok"
            fi
        fi
        ;;
    *)
        echo "Illegal Parameter <$1>"
        ;;
esac
